package core.notevault;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.Menu;
import com.google.android.material.snackbar.Snackbar;
import com.google.android.material.navigation.NavigationView;
import androidx.navigation.NavController;
import androidx.navigation.Navigation;
import androidx.navigation.ui.AppBarConfiguration;
import androidx.navigation.ui.NavigationUI;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.appcompat.app.AppCompatActivity;
import core.notevault.data.MusicDatabase;
import core.notevault.data.MusicNote;
import core.notevault.databinding.ActivityMainBinding;
import core.notevault.ui.metadatadialog.MetaDataDialog;

import java.io.*;
import java.nio.file.Files;

public class MainActivity extends AppCompatActivity implements MetaDataDialog.OnMetadataListener {

    private AppBarConfiguration mAppBarConfiguration;
    private ActivityMainBinding binding;

    private MusicDatabase musicDB;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        binding = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        setSupportActionBar(binding.appBarMain.toolbar);
        DrawerLayout drawer = binding.drawerLayout;
        NavigationView navigationView = binding.navView;
        // Passing each menu ID as a set of Ids because each
        // menu should be considered as top level destinations.
        mAppBarConfiguration = new AppBarConfiguration.Builder(
                R.id.nav_home, R.id.nav_gallery, R.id.nav_slideshow)
                .setOpenableLayout(drawer)
                .build();
        NavController navController = Navigation.findNavController(this, R.id.nav_host_fragment_content_main);
        NavigationUI.setupActionBarWithNavController(this, navController, mAppBarConfiguration);
        NavigationUI.setupWithNavController(navigationView, navController);

        musicDB = MusicDatabase.getDatabase(this);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    @Override
    public boolean onSupportNavigateUp() {
        NavController navController = Navigation.findNavController(this, R.id.nav_host_fragment_content_main);
        return NavigationUI.navigateUp(navController, mAppBarConfiguration)
                || super.onSupportNavigateUp();
    }

    @Override
    public void onMetadataEntered(Uri uri, String title, String composer, int year, String genre) {
        String path = saveImageInternally(uri);
        MusicNote musicNote = new MusicNote(title, path, composer, year, genre);

        // Anfordern der dauerhaften Lese-/Schreibberechtigungen für die URI

        new Thread(() -> {
            musicDB.musicNoteDao().insert(musicNote);
        }).start();
    }

    // Speichere eine Kopie des Bilds im internen Speicher
    private String saveImageInternally(Uri uri) {
        try (InputStream inputStream = getContentResolver().openInputStream(uri)) {
            File imageFile = new File(getFilesDir(), "saved_image_" + System.currentTimeMillis() + ".jpg");
            try (OutputStream outputStream = Files.newOutputStream(imageFile.toPath())) {
                byte[] buffer = new byte[4096];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
            }
            return imageFile.getAbsolutePath(); // Pfad speichern, um später darauf zuzugreifen
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }
}